#include <cmath>
#include "Math.hpp"
#include "raymath.h"

Color ColorLerp(Color c1, Color c2, float amount)
{
  Color ret{0};
  ret.r = Clamp(Lerp(c1.r, c2.r, amount), 0, 255);
  ret.g = Clamp(Lerp(c1.g, c2.g, amount), 0, 255);
  ret.b = Clamp(Lerp(c1.b, c2.b, amount), 0, 255);
  ret.a = Clamp(Lerp(c1.a, c2.a, amount), 0, 255);
  return ret;
}

Vector2 calculateVector(float rotation, float offSet, float len)
{
  float angle = ((rotation)*PI) / 180.0f;
  angle += offSet;
  return {
      .x = len * std::sin(angle),
      .y = len * std::cos(angle)};
}

Rectangle calculateRotatedRectangle(Vector2 center, float rotation, float width, float height)
{
  Rectangle ret = {0, 0, width, height};

  width /= 2;                                                      // a
  height /= 2;                                                     // b
  float len = std::sqrt(std::pow(width, 2) + std::pow(height, 2)); // c
  float offSet = std::atan2(width, height);                        // add angle alfa

  Vector2 vec = calculateVector(rotation + 180.0f, offSet, len);

  ret.x = vec.x + center.x;
  ret.y = vec.y + center.y;
  return ret;
}